﻿using System;
using System.Xml;

namespace PdpRtuDownloader.Database
{
    /// <summary>
    /// Local database gateway object.
    /// </summary>
    public abstract class DbGateway<T> where T: new()
    {
        #region Abstract methods

        /// <summary>
        /// Loads a domain object from an XML node.
        /// </summary>
        /// <param name="domainObject">Domain object.</param>
        /// <param name="node">Node.</param>
        public abstract void LoadFromXml(T domainObject, XmlNode node);

        /// <summary>
        /// Saves a domain object into an XML node.
        /// </summary>
        /// <param name="domainObject">Domain object.</param>
        /// <param name="node">Node.</param>
        public abstract void SaveToXml(T domainObject, XmlNode node);

        /// <summary>
        /// Compares two domain objects.
        /// </summary>
        /// <returns>Returns <c>true</c> if both objects have equal primary keys, otherwise <c>false</c>.</returns>
        /// <param name="domainObjectA">Domain object a.</param>
        /// <param name="domainObjectB">Domain object b.</param>
        public abstract bool PkEquals(T domainObjectA, T domainObjectB);

        /// <summary>
        /// Gets the name of the XML element which is the root of all domain objects.
        /// </summary>
        /// <returns>The root element name.</returns>
        public abstract string GetRootElementName();

        /// <summary>
        /// Gets the name of an XML element of a domain object.
        /// </summary>
        /// <returns>The element name.</returns>
        public abstract string GetElementName();

        #endregion

        #region Methods

        /// <summary>
        /// Gets the XML element which is the root of all domain objects.
        /// </summary>
        /// <returns>The root element.</returns>
        /// <param name="doc">Document.</param>
        public virtual XmlNode GetRootElement(XmlDocument doc)
        {
            XmlNode eRoot = doc[Database.rootElementName];
            XmlNode eRtuConfigs;
            string rootElementName = GetRootElementName();

            if (eRoot[rootElementName] != null)
            {
                eRtuConfigs = eRoot[rootElementName];
            }
            else
            {
                eRtuConfigs = doc.CreateElement(rootElementName);
                eRoot.AppendChild(eRtuConfigs);
            }

            return eRtuConfigs;
        }

        /// <summary>
        /// Inserts domain objects into database.
        /// </summary>
        /// <param name="database">Database.</param>
        /// <param name="domainObjects">Domain objects.</param>
        public void Insert(Database database, T[] domainObjects)
        {
            // Load the XML document.
            XmlDocument doc = database.LoadDocument();

            // Get the root of the domain object.
            XmlNode eRoot = GetRootElement(doc);

            // Retrieve element name to be a child of the root.
            string elementName = GetElementName();

            // Loop through all domain objects.
            foreach (T domainObject in domainObjects)
            {
                // Verify unique primary key values.
                foreach (XmlNode child in eRoot.ChildNodes)
                {
                    if (child.Name == elementName)
                    {
                        T testObject = new T();
                        LoadFromXml(testObject, child);
                        if (PkEquals(testObject, domainObject))
                        {
                            throw new DbException("Duplicate primary key.");
                        }
                    }
                }

                // Insert the domain objects.
                XmlNode eT = doc.CreateElement(elementName);
                SaveToXml(domainObject, eT);
                eRoot.AppendChild(eT);
            }

            // Save the XML document.
            database.SaveDocument(doc);
        }

        /// <summary>
        /// Inserts a domain object into database.
        /// </summary>
        /// <param name="database">Database.</param>
        /// <param name="domainObject">Domain object.</param>
        public void Insert(Database database, T domainObject)
        {
            Insert(database, new T[] { domainObject });
        }

        /// <summary>
        /// Updates domain objects in the database.
        /// </summary>
        /// <param name="database">Database.</param>
        /// <param name="domainObjects">Domain objects.</param>
        public void Update(Database database, T[] domainObjects)
        {
            // Load the XML document.
            XmlDocument doc = database.LoadDocument();

            // Get the root of the domain object.
            XmlNode eRoot = GetRootElement(doc);

            // Retrieve element name to be a child of the root.
            string elementName = GetElementName();

            // Loop through domain objects.
            foreach (T domainObject in domainObjects)
            {
                // Find the element to be updated.
                XmlNode eT = null;
                foreach (XmlNode child in eRoot.ChildNodes)
                {
                    if (child.Name == elementName)
                    {
                        T testObject = new T();
                        LoadFromXml(testObject, child);
                        if (PkEquals(testObject, domainObject))
                        {
                            eT = child;
                            break;
                        }
                    }
                }

                if (eT == null)
                {
                    throw new DbException("Cannot find the XML element to update.");
                }

                // Update the XML element.
                eT.RemoveAll();
                SaveToXml(domainObject, eT);
            }

            // Save the XML document.
            database.SaveDocument(doc);
        }

        /// <summary>
        /// Updates a domain object in the database.
        /// </summary>
        /// <param name="database">Database.</param>
        /// <param name="domainObject">Domain object.</param>
        public void Update(Database database, T domainObject)
        {
            Update(database, new T[] { domainObject });
        }

        /// <summary>
        /// Deletes domain objects from the database.
        /// </summary>
        /// <param name="database">Database.</param>
        /// <param name="domainObjects">Domain objects.</param>
        public void Delete(Database database, T[] domainObjects)
        {
            // Load the XML document.
            XmlDocument doc = database.LoadDocument();

            // Get the root of the domain object.
            XmlNode eRoot = GetRootElement(doc);

            // Retrieve element name to be a child of the root.
            string elementName = GetElementName();

            // Loop through domain objects.
            foreach (T domainObject in domainObjects)
            {
                // Find the element to be updated.
                XmlNode eT = null;
                foreach (XmlNode child in eRoot.ChildNodes)
                {
                    if (child.Name == elementName)
                    {
                        T testObject = new T();
                        LoadFromXml(testObject, child);
                        if (PkEquals(testObject, domainObject))
                        {
                            eT = child;
                            break;
                        }
                    }
                }

                if (eT == null)
                {
                    throw new DbException("Cannot find the XML element to delete.");
                }

                // Remove the child of the domain object.
                eRoot.RemoveChild(eT);
            }

            // Save the XML document.
            database.SaveDocument(doc);
        }

        /// <summary>
        /// Deletes a domain object from the database.
        /// </summary>
        /// <param name="database">Database.</param>
        /// <param name="domainObject">Domain object.</param>
        public void Delete(Database database, T domainObject)
        {
            Delete(database, new T[] { domainObject });
        }

        #endregion
    }
}

