﻿using System;
using System.IO;
using System.IO.Compression;
using tar_cs;

namespace PdpRtuDownloader.Measurement
{
    /// <summary>
    /// Loading of measurement data from a local archive.
    /// </summary>
    public class Loader
    {
        /// <summary>
        /// Trims all characters from the start and from the end that are not numbers or a decimal separator.
        /// </summary>
        /// <returns>The number.</returns>
        /// <param name="str">String.</param>
        private static string TrimNumber(string str)
        {
            const char decimalSeparator = '.';
            int start = -1, stop = -1;

            for (int i = 0; i < str.Length; i++)
            {
                if (char.IsDigit(str[i]) || str[i] == decimalSeparator)
                {
                    if (start == -1)
                    {
                        start = i;
                    }
                    stop = i + 1;
                }
            }

            if (stop >= start)
            {
                return str.Substring(start, stop - start);
            }
            else
            {
                return string.Empty;
            }
        }

        /// <summary>
        /// Parses measurement metadata from a stream.
        /// </summary>
        /// <returns>The metadata.</returns>
        /// <param name="stream">Stream.</param>
        private MeasurementMetadata ParseMetadata(Stream stream)
        {
            MeasurementMetadata ret = new MeasurementMetadata();

            using (StreamReader reader = new StreamReader(stream))
            {
                string line;
                while ((line = reader.ReadLine()) != null)
                {
                    string[] parts = line.Split('=');
                    if (parts.Length == 2)
                    {
                        string param = parts[0].Trim();
                        string value = parts[1].Trim();

                        switch (param)
                        {
                            case "STATUS":

                                switch (value)
                                {
                                    case "OK":
                                        ret.Status = MeasurementStatus.Ok;
                                        break;
                                    case "ERROR(1)":
                                        ret.Status = MeasurementStatus.Error1;
                                        break;
                                    case "ERROR(2)":
                                        ret.Status = MeasurementStatus.Error2;
                                        break;
                                    case "ERROR(3)":
                                        ret.Status = MeasurementStatus.Error3;
                                        break;
                                    case "ERROR(4)":
                                        ret.Status = MeasurementStatus.Error4;
                                        break;
                                    case "ERROR(5)":
                                        ret.Status = MeasurementStatus.Error5;
                                        break;
                                }
                                break;
                            case "CH1RANGE":
                                ret.Ch1Range = Convert.ToInt32(TrimNumber(value));
                                break;
                            case "CH2RANGE":
                                ret.Ch2Range = Convert.ToInt32(TrimNumber(value));
                                break;
                            case "CH3RANGE":
                                ret.Ch3Range = Convert.ToInt32(TrimNumber(value));
                                break;
                            case "CH4RANGE":
                                ret.Ch4Range = Convert.ToInt32(TrimNumber(value));
                                break;
                            case "INTTEMP1":
                                ret.IntTemp1 = Convert.ToDouble(TrimNumber(value));
                                break;
                            case "EXTTEMP1":
                                ret.ExtTemp1 = Convert.ToDouble(TrimNumber(value));
                                break;
                            case "PWR":
                                ret.Pwr = Convert.ToDouble(TrimNumber(value));
                                break;
                            case "BATT":
                                ret.Batt = Convert.ToDouble(TrimNumber(value));
                                break;
                            case "AKU":
                                ret.Aku = Convert.ToDouble(TrimNumber(value));
                                break;
                            case "EXTTEMP2":
                                ret.ExtTemp2 = Convert.ToDouble(TrimNumber(value));
                                break;
                            case "HUMIDITY1":
                                ret.Humidity1 = Convert.ToDouble(TrimNumber(value));
                                break;
                        }
                    }
                }
            }

            return ret;
        }

        /// <summary>
        /// Parses peak classification data from a stream.
        /// </summary>
        /// <returns>The peack classification.</returns>
        /// <param name="stream">Stream.</param>
        private PeakClassification ParsePeakClassification(Stream stream)
        {
            int[] peaks = new int[] { -1, -1, -1 };
            float result = -1;
            int pos = 0;

            using (StreamReader reader = new StreamReader(stream))
            {
                string line;
                while ((line = reader.ReadLine()) != null)
                {
                    if (line.StartsWith("#Peaks") || line.StartsWith("Result"))
                    {
                        string[] parts = line.Split(':');
                        if (parts.Length == 2)
                        {
                            string param = parts[0];
                            string value = parts[1];
                            string num = TrimNumber(value);

                            if (param == "#Peaks")
                            {
                                int val;
                                if (int.TryParse(num, out val))
                                {
                                    peaks[pos++] = val;
                                }
                            }
                            else if (param == "Result")
                            {
                                float val;
                                if (float.TryParse(num, out val))
                                {
                                    result = val;
                                }
                            }
                        }
                    }
                }
            }

            PeakClassification ret = new PeakClassification();
            ret.Peaks1 = peaks[0];
            ret.Peaks2 = peaks[1];
            ret.Peaks3 = peaks[2];
            ret.Result = result;
            return ret;
        }

        /// <summary>
        /// Reads and parses a measurement archive.
        /// </summary>
        /// <returns>The measurement data.</returns>
        /// <param name="measurement">Measurement.</param>
        /// <param name="localDirectory">Local directory.</param>
        public MeasurementData LoadMeasurementData(string measurement, string localDirectory)
        {
            // Prepare the result.
            MeasurementData ret = new MeasurementData();
            ret.Timestamp = measurement;

            // Input files.
            string archiveInputFileName = Path.Combine(localDirectory, measurement + Constants.measurePostfix);
            string peakClassInputFileName = Path.Combine(localDirectory, measurement + Constants.peakClassDataPostfix);

            // Files to be searched in the archive.
            string ch1FileName = string.Format("{0}_1.bin", measurement);
            string ch2FileName = string.Format("{0}_2.bin", measurement);
            string ch3FileName = string.Format("{0}_3.bin", measurement);
            string ch4FileName = string.Format("{0}_4.bin", measurement);
            string metaFileName = string.Format("{0}.txt", measurement);
            string peakClassFileName = string.Format("{0}_pc.txt", measurement);

            // Read .tar.gz archive and process the decompressed files.
            using (Stream inputStream = File.OpenRead(archiveInputFileName))
            {
                using (GZipStream gZipStream = new GZipStream(inputStream, CompressionMode.Decompress))
                {
                    TarReader tarReader = new TarReader(gZipStream);

                    while (tarReader.MoveNext(true))
                    {
                        MemoryStream memStream = new MemoryStream();
                        tarReader.Read(memStream);
                        byte[] data = memStream.ToArray();
                    
                        if (tarReader.FileInfo.FileName == ch1FileName)
                        {
                            ret.Ch1Data = data;
                        }
                        else if (tarReader.FileInfo.FileName == ch2FileName)
                        {
                            ret.Ch2Data = data;
                        }
                        else if (tarReader.FileInfo.FileName == ch3FileName)
                        {
                            ret.Ch3Data = data;
                        }
                        else if (tarReader.FileInfo.FileName == ch4FileName)
                        {
                            ret.Ch4Data = data;
                        }
                        else if (tarReader.FileInfo.FileName == metaFileName)
                        {
                            ret.Metadata = ParseMetadata(new MemoryStream(data));
                        }
                        else if (tarReader.FileInfo.FileName == peakClassFileName)
                        {
                            ret.PeakClassification = ParsePeakClassification(new MemoryStream(data));   
                        }
                    }
                }
            }

            // Problem with classification data, try to read not from archive.
            if (ret.PeakClassification == null || ret.PeakClassification.Result == -1)
            {
                if (File.Exists(peakClassInputFileName))
                {
                    Stream stream = File.OpenRead(peakClassInputFileName);
                    ret.PeakClassification = ParsePeakClassification(stream);
                }
            }

            return ret;
        }
    }
}

