﻿using System;
using System.Collections.Generic;
using System.Net;
using System.IO;

namespace PdpRtuDownloader.RtuStation
{
    /// <summary>
    /// Basic communication with an RTU station, downloading measurement archives and peak classifier data files.
    /// </summary>
    public class RtuStation
    {
        #region Members

        private Database.DomainObjects.RtuConfig rtuConfig;
        private string localDirectory;

        #endregion

        #region Properties

        /// <summary>
        /// Gets the RTU station configuration.
        /// </summary>
        /// <value>The rtu config.</value>
        public Database.DomainObjects.RtuConfig RtuConfig
        {
            get { return rtuConfig; }
        }

        /// <summary>
        /// Gets or sets the local directory where the measured files of the station will be downloaded.
        /// </summary>
        /// <value>The local directory.</value>
        public string LocalDirectory
        {
            get { return localDirectory; }
            set { localDirectory = value; }
        }

        #endregion

        #region Constructors

        public RtuStation(Database.DomainObjects.RtuConfig rtuConfig)
        {
            this.rtuConfig = rtuConfig;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Tests whether the specified string ends with a postfix.
        /// </summary>
        /// <returns><c>true</c> if this instance has postfix the specified str postfix; otherwise, <c>false</c>.</returns>
        /// <param name="str">String.</param>
        /// <param name="postfix">Postfix.</param>
        private static bool HasPostfix(string str, string postfix)
        {
            if (str.Length >= postfix.Length)
            {
                return str.Substring(str.Length - postfix.Length) == postfix;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// Gets all measurements, i.e., all file names with postfix .tar.gz without the postfix.
        /// </summary>
        /// <returns>The measurements.</returns>
        public IEnumerable<string> ListMeasurements()
        {
            try
            {
                List<string> ret = new List<string>();

                Uri target = new Uri (rtuConfig.Uri);
                FtpWebRequest request = (FtpWebRequest)WebRequest.Create (target);

                request.Method = WebRequestMethods.Ftp.ListDirectory;
                request.Credentials = new NetworkCredential (rtuConfig.Login, rtuConfig.Password);

                WebResponse response = request.GetResponse ();

                Stream stream = response.GetResponseStream ();
                StreamReader reader = new StreamReader (stream);

                string line;
                while ((line = reader.ReadLine()) != null)
                {
                    if (HasPostfix(line, Constants.measurePostfix))
                    {
                        string lineWithoutPostfix = line.Substring(0, line.Length - Constants.measurePostfix.Length);
                        ret.Add(lineWithoutPostfix);
                    }
                }

                stream.Close ();

                return ret;
            }
            catch (Exception ex)
            {
                throw new RsException("Could not list measurements.", ex);
            }
        }

        /// <summary>
        /// Tests whether the file exists in RTU.
        /// </summary>
        /// <returns><c>true</c>, if file exists exists, <c>false</c> otherwise.</returns>
        /// <param name="fileName">File name.</param>
        private bool FtpFileExists(string fileName)
        {
            try
            {
                string srcUri = rtuConfig.Uri + fileName;

                Uri target = new Uri(srcUri);
                FtpWebRequest request = (FtpWebRequest)WebRequest.Create(target);

                request.Method = WebRequestMethods.Ftp.GetFileSize;
                request.Credentials = new NetworkCredential (rtuConfig.Login, rtuConfig.Password);
                WebResponse response = request.GetResponse();
                Stream stream = response.GetResponseStream();
                stream.Close();

                return true;
            }
            catch (WebException)
            {
                return false;
            }
            catch (Exception ex)
            {
                throw new RsException("Could check whether the file exists.", ex);
            }
        }

        /// <summary>
        /// Downloads a single file.
        /// </summary>
        /// <param name="fileName">File name.</param>
        private void FtpDownloadFile(string fileName)
        {
            try
            {
                const int bufferSize = 1024;
                byte[] buffer = new byte[bufferSize];
                int bytesRead;

                string srcUri = rtuConfig.Uri + fileName;
                string dstFileName = Path.Combine(localDirectory, fileName);

                Uri target = new Uri(srcUri);
                FtpWebRequest request = (FtpWebRequest)WebRequest.Create(target);

                request.Method = WebRequestMethods.Ftp.DownloadFile;
                request.Credentials = new NetworkCredential (rtuConfig.Login, rtuConfig.Password);

                WebResponse response = request.GetResponse();

                Stream srcStream = response.GetResponseStream();
                FileStream dstStream = new FileStream(dstFileName, FileMode.Create, FileAccess.Write);

                bytesRead = srcStream.Read(buffer, 0, bufferSize);
                while (bytesRead > 0)
                {
                    dstStream.Write(buffer, 0, bytesRead);
                    bytesRead = srcStream.Read(buffer, 0, bufferSize);
                }

                dstStream.Flush();

                srcStream.Close();
                dstStream.Close();
            }
            catch (Exception ex)
            {
                throw new RsException("Could not download file.", ex);
            }
        }

        /// <summary>
        /// Downloads the specified measurements.
        /// </summary>
        /// <param name="measurements">Measurements.</param>
        public void DownloadMeasurements(IEnumerable<string> measurements)
        {
            try
            {
                foreach (string measurement in measurements)
                {
                    // Download the measurement archive.
                    string measureFileName = measurement + Constants.measurePostfix;
                    FtpDownloadFile(measureFileName);

                    // Download peak classifier data separately, if exist.
                    string peakClassDataFileName = measurement + Constants.peakClassDataPostfix;
                    if (FtpFileExists(peakClassDataFileName))
                    {
                        FtpDownloadFile(peakClassDataFileName);
                    }
                }
            }
            catch (Exception ex)
            {
                throw new RsException("Could not download measurements.", ex);
            }
        }

        #endregion
    }
}

