﻿using System;

namespace PdpRtuDownloader.WebService
{
    /// <summary>
    /// Commmunication with the web service.
    /// </summary>
    public class WebService
    {
        private string uri;

        /// <summary>
        /// Gets the web service URI.
        /// </summary>
        /// <value>The URI.</value>
        public string Uri
        {
            get { return uri; }
        }

        public WebService(string uri)
        {
            this.uri = uri;
        }

        /// <summary>
        /// Returns true if the measurement of the station already exists in the web service database.
        /// </summary>
        /// <returns><c>true</c>, if measurement exists was checked, <c>false</c> otherwise.</returns>
        /// <param name="measurement">Measurement.</param>
        /// <param name="idStation">Identifier station.</param>
        public bool CheckMeasurementExists(string measurement, int idStation)
        {
            pdpattern.ServiceDb service = new PdpRtuDownloader.pdpattern.ServiceDb(uri);
            int result;
            bool resultIdSpecified;
            service.GetMeasurementId(measurement, idStation, true, out result, out resultIdSpecified);
            return resultIdSpecified && result != -1;
        }

        /// <summary>
        /// Converts the metadata of MeasurementData to web service compatible PdmetadataType type.
        /// </summary>
        /// <returns>The ws metadata.</returns>
        /// <param name="data">Data.</param>
        private static pdpattern.PdmetadataType GetWsMetadata(Measurement.MeasurementData data)
        {
            Measurement.MeasurementMetadata meta = data.Metadata;

            pdpattern.PdmetadataType ret = new PdpRtuDownloader.pdpattern.PdmetadataType();
            ret.Aku = (float)meta.Aku;
            ret.AkuSpecified = true;

            ret.Batt = (float)meta.Batt;
            ret.BattSpecified = true;

            ret.ExtTemp1 = (float)meta.ExtTemp1;
            ret.ExtTemp1Specified = true;

            ret.ExtTemp2 = (float)meta.ExtTemp2;
            ret.ExtTemp2Specified = true;

            ret.Humidity1 = (float)meta.Humidity1;
            ret.Humidity1Specified = true;

            ret.Pwr = (float)meta.Pwr;
            ret.PwrSpecified = true;

            ret.Status = (int)meta.Status;
            ret.StatusSpecified = true;

            ret.IndicatorFftU1 = false;
            ret.IndicatorFftU1Specified = true;

            ret.IndicatorFftU2 = false;
            ret.IndicatorFftU2Specified = true;

            ret.IndicatorFftU3 = false;
            ret.IndicatorFftU3Specified = true;

            ret.Timestamp = data.Timestamp;

            ret.IdMeasurement = -1;
            ret.IdMeasurementSpecified = true;

            return ret;
        }

        /// <summary>
        /// Uploads the measured data into the web service. Validates whether the data were uploaded correctly.
        /// </summary>
        /// <param name="measurementData">Measurement data.</param>
        /// <param name="stationId">Station identifier.</param>
        public void UploadMeasurementData(Measurement.MeasurementData measurementData, int stationId)
        {
            byte[] data1 = measurementData.Ch1Data;
            byte[] data2 = measurementData.Ch2Data;
            byte[] data3 = measurementData.Ch3Data;

            string data1Str = Convert.ToBase64String(data1);
            string data2Str = Convert.ToBase64String(data2);
            string data3Str = Convert.ToBase64String(data3);

            int samplingRate = measurementData.SamplingRate;
            int voltageRate = measurementData.Metadata.Ch1Range;

            pdpattern.ServiceDb service = new PdpRtuDownloader.pdpattern.ServiceDb(uri);
            pdpattern.PdmetadataType meta = GetWsMetadata(measurementData);

            int idMeasurement;
            bool idMeasurementSpecified;

            Console.WriteLine("IDStation = {0}, {1}", stationId, measurementData);

            meta.IdStation = stationId;
            meta.IdStationSpecified = true;

            service.AddMeasurementSampling(meta, data1Str, data2Str, data3Str,
                samplingRate, true, voltageRate, true, out idMeasurement, out idMeasurementSpecified);
                
            if (!idMeasurementSpecified || idMeasurement <= 0)
            {
                throw new WsException("Unexpected result of uploading a measurement.");
            }

            if (measurementData.FourPhases)
            {
                byte[] data4 = measurementData.Ch4Data;
                string data4Str = Convert.ToBase64String(data4);

                pdpattern.PdpatternType pdPattern = new PdpRtuDownloader.pdpattern.PdpatternType();
                pdPattern.data = data4Str;

                pdPattern.idMeasurement = idMeasurement;
                pdPattern.idMeasurementSpecified = true;

                pdPattern.phase = 4;
                pdPattern.phaseSpecified = true;

                pdPattern.samplingRate = samplingRate;
                pdPattern.samplingRateSpecified = true;

                pdPattern.voltageRange = voltageRate;
                pdPattern.voltageRangeSpecified = true;

                bool result;
                bool resultSpecified;
                service.AddPdpattern(pdPattern, out result, out resultSpecified);
            }

            Console.WriteLine("Measurement {0} uploaded", measurementData.Timestamp);
        }
    }
}

